<?php

namespace Drupal\Tests\contact_permissions\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\user\Traits\UserCreationTrait;

/**
 * Tests the permission provided by contact_permissions module.
 *
 * @package Drupal\Tests\contact_permissions\Functional
 *
 * @group contact_permissions
 */
class ContactPermissionsTest extends BrowserTestBase {

  use UserCreationTrait;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'contact_permissions_test',
  ];

  /**
   * A user with the contactable role.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $contactable;

  /**
   * A user with the noncontactable role.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $normalUser;

  /**
   * A user with the administrator role.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $admin;

  /**
   * {@inheritdoc}
   */
  public function setUp() {
    parent::setUp();

    $contactable = $this->drupalCreateUser();
    $contactable->addRole('contactable');
    $contactable->save();
    $this->contactable = $contactable;

    $normal_user = $this->drupalCreateUser();
    $normal_user->addRole('noncontactable');
    $normal_user->save();
    $this->normalUser = $normal_user;

    $this->admin = $this->drupalCreateUser([], NULL, TRUE);
  }

  /**
   * Tests contact permissions.
   *
   * @throws \Behat\Mink\Exception\ExpectationException
   */
  public function testContactPermissions() {
    // Assert that the roles defined by view_profiles_perms_test module get
    // their permissions generated and appear correctly in the UI.
    $assert = $this->assertSession();
    $this->drupalLogin($this->admin);
    $this->drupalGet('admin/people/permissions');
    $assert->pageTextContains('Have a personal contact form');
    $assert->checkboxChecked('contactable[have a personal contact form]');
    $assert->checkboxNotChecked('anonymous[have a personal contact form]');
    $assert->checkboxNotChecked('authenticated[have a personal contact form]');

    // Tests for asserting access to personal contact forms based on our
    // permission.
    // - Contactable role has 'have a personal contact form'
    // By default Drupal only assigns 'have a personal contact form' to the
    // administrator role.
    // Assert users can't access normal user contact form.
    $this->drupalLogin($this->contactable);
    $this->drupalGet('user/' . $this->normalUser->id() . '/contact');
    $assert->statusCodeEquals(403);

    // Assert contactable can configure their contact form.
    $this->drupalGet('user/' . $this->contactable->id() . '/edit');
    $assert->pageTextContains('Contact settings');
    $assert->pageTextContains('Personal contact form');
    $assert->checkboxChecked('contact');

    // Assert users can access contactable contact form.
    $this->drupalLogin($this->normalUser);
    $this->drupalGet('user/' . $this->contactable->id() . '/contact');
    $assert->statusCodeEquals(200);

    // Assert users without the permission cannot configure their contact form.
    $this->drupalGet('user/' . $this->normalUser->id() . '/edit');
    $assert->pageTextNotContains('Contact settings');
    $assert->pageTextNotContains('Personal contact form');
    // Ensure the user edit form can be saved fine.
    $this->submitForm([], 'Save', 'user-form');
    $assert->pageTextContains('The changes have been saved.');

    // Assert users with more than one role, and only one with access.
    $user = $this->drupalCreateUser();
    $user->addRole('contactable');
    $user->addRole('noncontactable');
    $user->save();
    $this->drupalGet('user/' . $user->id() . '/contact');
    $assert->statusCodeEquals(200);

    // Assert users can't visit their own contact page.
    $this->drupalGet('user/' . $this->normalUser->id() . '/contact');
    $assert->statusCodeEquals(403);

    // An inactive/blocked user's contact page should never be affected by our
    // permissions.
    $this->contactable->block();
    $this->contactable->save();
    $this->drupalGet('user/' . $this->contactable->id() . '/contact');
    $assert->statusCodeEquals(403);

    // Assert users can still deactivate their personal contact forms.
    $this->contactable->activate();
    $this->contactable->save();
    $this->drupalLogin($this->contactable);
    $edit = [
      'contact' => FALSE,
    ];
    $this->drupalPostForm('user/' . $this->contactable->id() . '/edit', $edit, 'Save');
    $assert->pageTextContains('The changes have been saved.');
    // Assert users can no longer access contactable contact form.
    $this->drupalLogin($this->normalUser);
    $this->drupalGet('user/' . $this->contactable->id() . '/contact');
    $assert->statusCodeEquals(403);
  }

}
